package com.tenlionsoft.aimz_k.page.activity

import android.annotation.SuppressLint
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.util.Log
import android.view.View
import android.view.WindowManager
import androidx.core.content.ContextCompat
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import androidx.recyclerview.widget.LinearLayoutManager
import com.atwa.filepicker.core.FilePicker
import com.tenlionsoft.aimz_k.R
import com.tenlionsoft.aimz_k.databinding.ActivityChatBinding
import com.tenlionsoft.aimz_k.model.PickerType
import com.tenlionsoft.aimz_k.viewmodel.ChatPageViewModel
import com.tenlionsoft.baselib.base.BaseActivity
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.widget.SoftKeyBoardListener
import com.tenlionsoft.baselib.widget.wheel.WheelView


/**
 * 聊天页面
 */
class ChatActivity : BaseActivity() {
    private lateinit var mBinding: ActivityChatBinding
    private lateinit var mLocalReceiver: LocalReceiver
    var chatPageViewModel: ChatPageViewModel? = null
    private val filePicker = FilePicker.getInstance(this)
    private var mContentHeight: Int = 0

    override fun bindView() {
        mBinding = DataBindingUtil.setContentView(this, R.layout.activity_chat);
        window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
        window.statusBarColor =
            ContextCompat.getColor(this, com.tenlionsoft.baselib.R.color.chat_page_bg)

        val fromId = intent.getStringExtra("fromId") //传递过来的接收人
        val name = intent.getStringExtra("name")
        if (name.isNullOrEmpty()) {
            mBinding.tvTitle.text = "临时客户"
        }

        chatPageViewModel = ViewModelProvider(this, object : ViewModelProvider.Factory {
            override fun <T : ViewModel> create(modelClass: Class<T>): T {
                return ChatPageViewModel(
                    fromId!!, SpUtils.getId(), this@ChatActivity.applicationContext
                ) as T
            }
        })[ChatPageViewModel::class.java]

        mBinding.ivBack.setOnClickListener { finish(); }
        mBinding.viewModel = chatPageViewModel
        mBinding.lifecycleOwner = this
        //选择emoji
        mBinding.rpvEmoji.setOnEmojiPickedListener {
            mBinding.etMsg.append(it.emoji)
        }

        //显示/隐藏软键盘
//        chatPageViewModel!!.showSoftKeyboard.observe(this) {
//            if (it) {
//                showSoftKeyBoard(mBinding.etMsg)
//                chatPageViewModel!!.showChooseLayout.value = false
//                chatPageViewModel!!.showEmojiLayout.value = false
//                chatPageViewModel!!.showReplyLayout.value = false
//                val layoutManager = mBinding.rlvChats.layoutManager as LinearLayoutManager
//                val itemCount = layoutManager.itemCount
//                mBinding.rlvChats.smoothScrollToPosition(itemCount - 1)
//            } else {
//                hideSoftKeyboard()
//            }
//        }
//        mBinding.rlContent.setOnClickListener {
//            Log.e("ChatActivity", "bindView: 点击")
//            chatPageViewModel!!.showChooseLayout.value = false
//            chatPageViewModel!!.showEmojiLayout.value = false
//            chatPageViewModel!!.showReplyLayout.value = false
//            hideSoftKeyboard()
//        }
        mBinding.wvView.setTextSize(14F, isSp = true)
        mBinding.wvView.setAutoFitTextSize(true)
        mBinding.wvView.setOnItemSelectedListener(object : WheelView.OnItemSelectedListener {
            override fun onItemSelected(wheelView: WheelView, data: Any, position: Int) {
                mBinding.etMsg.setText(data.toString())
            }
        })
        chatPageViewModel!!.chooseType.observe(this) {
            when (it) {
                PickerType.TYPE_PIC -> {
                    filePicker.pickMultiImage { list ->
                        if (list.isNotEmpty()) {
                            chatPageViewModel!!.uploadImages(list)
                        }
                    }
                }

                PickerType.TYPE_VIDEO -> {
                    filePicker.pickVideo { mate ->
                        if (mate != null) {
                            chatPageViewModel!!.uploadVideo(mate)
                        }
                    }
                }

                PickerType.TYPE_FILE -> {
                    filePicker.pickFile { meta ->
                        if (meta != null) {
                            chatPageViewModel!!.uploadFiles(meta)
                        }
                    }
                }

                else -> {}
            }
        }
        chatPageViewModel!!.scrollListToBottom.observe(this) {
            Log.e("TAG", "bindView:滚动    ${it} ")
            if (it) {
                val layoutManager = mBinding.rlvChats.layoutManager as LinearLayoutManager
                val itemCount = layoutManager.itemCount
                //滚动到底部

                mBinding.rlvChats.postDelayed({
                    layoutManager.scrollToPositionWithOffset(
                        itemCount - 1,
                        0
                    )
                }, 50)
                chatPageViewModel!!.scrollListToBottom.value = false
            }
        }
        mBinding.llBottomLayout.visibility = View.VISIBLE
        mBinding.rlBottom.visibility = View.GONE
        mBinding.rlvChats.viewTreeObserver.addOnGlobalLayoutListener {
            mContentHeight = mBinding.rlvChats.height
        }
        SoftKeyBoardListener().setChangeListener(this@ChatActivity,
            { h ->
                mBinding.llBottomLayout.visibility = View.VISIBLE
                mBinding.rlBottom.visibility = View.VISIBLE
                //重置layout高度
                val layoutParams = mBinding.rlvChats.layoutParams
                layoutParams.height = mContentHeight - h
                mBinding.rlvChats.layoutParams = layoutParams
                val layoutManager = mBinding.rlvChats.layoutManager as LinearLayoutManager
                val itemCount = layoutManager.itemCount
                //滚动到底部
                mBinding.rlvChats.postDelayed({
                    layoutManager.scrollToPositionWithOffset(
                        itemCount - 1,
                        0
                    )
                }, 100)


            },
            { _ ->
                val layoutParams = mBinding.rlvChats.layoutParams
                layoutParams.height = mContentHeight
                mBinding.rlvChats.layoutParams = layoutParams
                mBinding.llBottomLayout.visibility = View.VISIBLE
                mBinding.rlBottom.visibility = View.GONE

            })
        registerLocalReceiver()
    }


    @SuppressLint("UnspecifiedRegisterReceiverFlag")
    private fun registerLocalReceiver() {
        mLocalReceiver = LocalReceiver()
        val intentFilter = IntentFilter()
        intentFilter.addAction(ProjectConfig.A_S_MSG_RECEIVER)
        registerReceiver(mLocalReceiver, intentFilter)
    }

    private fun unRegisterLocalReceiver() {
        unregisterReceiver(mLocalReceiver)
    }

    override fun onDestroy() {
        super.onDestroy()
        unRegisterLocalReceiver()
    }

    inner class LocalReceiver : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            when (intent?.action) {
                ProjectConfig.A_S_MSG_RECEIVER -> {
                    val msg = intent.getStringExtra("msg")
                    Log.e("LocalReceiver", "onReceive: ${msg}")
                    chatPageViewModel!!.receiveMsg(msg)
                }//接收到信息
            }
        }
    }
}


