package com.tenlionsoft.aimz_k.page.fragments

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import com.tenlionsoft.aimz_k.R
import com.tenlionsoft.aimz_k.databinding.FragmentMsgBinding
import com.tenlionsoft.aimz_k.model.MsgCategoryBean
import com.tenlionsoft.aimz_k.page.activity.ChatActivity
import com.tenlionsoft.aimz_k.page.activity.MainActivity
import com.tenlionsoft.aimz_k.services.SocketService
import com.tenlionsoft.aimz_k.viewmodel.MsgViewModel
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.widget.LoadingDialog

class MsgFragment : Fragment(), MsgViewModel.OnItemClickListener,
    MainActivity.OnSocketConnectListener {
    private lateinit var mMsgBinding: FragmentMsgBinding
    private var mActivity: MainActivity? = null
    private var mLoading: LoadingDialog? = null;

    companion object {
        fun newInstance() = MsgFragment()
    }

    private val viewModel: MsgViewModel by viewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        mMsgBinding = DataBindingUtil.inflate(
            layoutInflater,
            R.layout.fragment_msg,
            container,
            false
        );
        mMsgBinding.llSearchLayout.llSearchLayout.setOnClickListener {
            startActivity(Intent(this@MsgFragment.context, ChatActivity::class.java))
        }
        mMsgBinding.msgViewModel = viewModel
        mMsgBinding.lifecycleOwner = this
        viewModel.onItemClickListener = this//条目点击
        initView();
        return mMsgBinding.root;
    }

    /**
     * 初始化页面
     */
    private fun initView() {
        mMsgBinding.srlMsgs.setOnLoadMoreListener { viewModel.loadMore() }
        mMsgBinding.srlMsgs.setOnRefreshListener { viewModel.refresh() }

        viewModel.isHasMore.observe(viewLifecycleOwner) {
            mMsgBinding.srlMsgs.setNoMoreData(it)
            mMsgBinding.srlMsgs.finishRefresh()
            mMsgBinding.srlMsgs.finishLoadMore()
        }
        viewModel.isLogining.observe(viewLifecycleOwner) {
            if (it) {
                mMsgBinding.llSearchLayout.tvSocketStatus.text = "登陆中..."
                mMsgBinding.llSearchLayout.pbSocketLoading.visibility = View.VISIBLE
                toStartService()
            } else {
                mMsgBinding.llSearchLayout.tvSocketStatus.visibility = View.GONE
                mMsgBinding.llSearchLayout.pbSocketLoading.visibility = View.GONE
            }
        }
        //登陆socket
        viewModel.doLoginWebsocket()
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is MainActivity) {
            mActivity = context
            mActivity!!.setOnSocketListener(this)
        }
    }

    //开启service
    private fun toStartService() {
        mActivity?.startService(Intent(mActivity!!, SocketService::class.java))
    }

    override fun onResume() {
        super.onResume()
        viewModel.refresh()
    }

    /**
     * 条目点击
     */
    override fun onItemClick(msgCategoryBean: MsgCategoryBean) {
        startActivity(
            Intent(mActivity, ChatActivity::class.java).putExtra(
                "fromId",
                msgCategoryBean.senderId
            )
        )
    }

    //监听socket连接状态
    override fun onListener(status: String) {
        when (status) {
            ProjectConfig.A_S_CONNECTED -> {
                viewModel.isLogining.value = false
            }//连接成功
            ProjectConfig.A_S_FAIL -> {
                viewModel.isLogining.value = true
            }//连接失败
            ProjectConfig.A_S_DISCONNECT -> {}//连接断开
        }
    }

    //接收到socket信息
    override fun onReceiverMsg(msg: String) {
        if (msg.isNotEmpty()) {
            viewModel.fromMsg(msg, mActivity!!.applicationContext)
        }
    }

}