package com.tenlionsoft.aimz_k.services

import android.annotation.SuppressLint
import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.Service
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Build
import android.os.IBinder
import android.util.Log
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.socket.WsManager
import com.tenlionsoft.baselib.contacts.NetConfig
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.ToastUtils


/**
 * Socket service
 */
class SocketService : Service(), WsManager.MsgCallBack {
    private var mWsManager: WsManager? = null
    private val gson = Gson()
    private lateinit var mLocalReceiver: LocalReceiver
    override fun onCreate() {
        super.onCreate()
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val channel = NotificationChannel(
                ProjectConfig.NOTIFY_CHANNEL_ID, "Ai秒著客服",
                NotificationManager.IMPORTANCE_DEFAULT
            )
            val manager = getSystemService(NotificationManager::class.java)
            manager.createNotificationChannel(channel)
        }
    }


    @SuppressLint("ForegroundServiceType")
    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
//        val builder: Notification.Builder
//        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
//            builder = Notification.Builder(this, ProjectConfig.NOTIFY_CHANNEL_ID)
//        } else {
//            builder = Notification.Builder(this)
//        }
//        builder.setContentTitle("Ai秒著")
//            .setContentText("聊天已经登陆")
//            .setSmallIcon(R.drawable.app_logo_small)
//        val notification: Notification = builder.build()
//        startForeground(startId, notification)

        this.startSocket();//开启Socket
        registerLocalReceiver()
        return START_STICKY
    }


    private fun registerLocalReceiver() {
        val intentFilter = IntentFilter()
        mLocalReceiver = LocalReceiver()
        intentFilter.addAction(ProjectConfig.A_S_MSG_SEND)//发送消息
        registerReceiver(mLocalReceiver, intentFilter)
    }

    /**
     * 开启链接socket
     */
    private fun startSocket() {
        if (mWsManager == null) {
            mWsManager = WsManager.Builder(applicationContext)
                .wsUrl(NetConfig.WS_URL + "?token=" + SpUtils.getToken())
                .needReconnect(true)
                .addCallBack(this)
                .build()
        }
        if (!mWsManager!!.isWsConnected()) {
            mWsManager!!.startConnect();
        }

    }

    private fun stopSocket() {
        if (mWsManager != null && mWsManager!!.isWsConnected()) {
            mWsManager!!.stopConnect();
            mWsManager = null;
        }
    }

    override fun onBind(intent: Intent?): IBinder? {
        return null;
    }

    override fun onDestroy() {
        stopSocket();
        unregisterReceiver(mLocalReceiver)
        super.onDestroy()
    }

    //发送消息
    inner class LocalReceiver : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            val isConnect = mWsManager?.isNetworkConnected(context)
            if (true == isConnect) {
                when (intent?.action) {
                    ProjectConfig.A_S_MSG_SEND -> {
                        val msgConvertBean = intent.getSerializableExtra("msgBean")
                        val msg = gson.toJson(msgConvertBean)
                        mWsManager?.sendMessage(msg)
                    }//发送消息
                }
            } else {
                ToastUtils.error("请检查网络")
            }

        }

    }

    //socket接收到信息
    override fun onCallBackMsg(str: String) {
        Log.e("SocketService", "接收信息:     ${str}")
        val intent = Intent(ProjectConfig.A_S_MSG_RECEIVER)
        intent.putExtra("msg", str)
        sendBroadcast(intent)
    }
}